@@ -1,7 +1,7 @@
 # /// script
 # dependencies = [
 #   "fire",
-#   "HS-TasNet",
+#   "HS-TasNet>=0.2.1",
 #   "wandb"
 # ]
 # ///
@@ -14,7 +14,6 @@ import fire
 import musdb
 from hs_tasnet import HSTasNet, Trainer
 
-@fire.Fire
 def train(
     small = False,
     stereo = False,
@@ -26,7 +25,9 @@ def train(
     wandb_run_name = None,
     split_dataset_for_eval = True,
     split_dataset_eval_frac = 0.05,
-    clear_folders = False
+    clear_folders = False,
+    use_full_musdb_dataset = False,
+    full_musdb_dataset_root = "./full-musdb-dataset"
 ):
 
     model = HSTasNet(
@@ -34,17 +35,16 @@ def train(
         stereo = stereo
     )
 
-    # the musdb dataset
-
-    mus = musdb.DB(download = True)
-
     # trainer
 
     from hs_tasnet import Trainer
 
     trainer = Trainer(
         model,
-        dataset = mus,
+        dataset = None,                                     # add your own
+        concat_musdb_dataset = True,                        # whether to concat the musdb dataset
+        use_full_musdb_dataset = use_full_musdb_dataset,    # whether to use sample musdb or full
+        full_musdb_dataset_root = full_musdb_dataset_root,
         batch_size = batch_size,
         max_steps = max_steps,
         max_epochs = max_epochs,
@@ -62,4 +62,5 @@ def train(
 # fire cli
 # --small for small model
 
-train()
+if __name__ == '__main__':
+    fire.Fire(train)
